#pragma NOIV	// tells compiler not to generate interrupt vectors
				// Note that we have to edit USBJmpTB if we wanna add
				// other interrupts later

/*=============================================================================
	File:      SyncBulk.c
	Contents:   Hooks required to implement USB peripheral function.
	Comments:	Original file was given by a Cypress example, this
				one is edited only so there might me reserved rights:

				Copyright (c) 2000 Cypress Semiconductor All rights reserved
=============================================================================*/

#include "fx2.h"
#include "fx2regs.h"			// includes FX2 register adressing
#include "fx2sdly.h"            // includes SYNCDELAY macro

/*=============================================================================
	Pre-Processor defines:   	
=============================================================================*/
// nothing to do here 

/*=============================================================================
	Global variables:
	External variables which belong to the Anchor USB Framework and are found
	in FW.c   	
=============================================================================*/

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;			   	// see TRM Chapter 6 Power Management
extern BOOL Rwuen;				// see TRM Chapter 6 Power Management
extern BOOL Selfpwr;			// see TRM Chapter 6 Power Management

BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings


// Some PortA settings (mostly defaults if SL FIFO enabled by IFCONFIG = 0x03
sbit PA0 = IOA ^ 0;	// is alt. function INT0# see TRM Chapter 4
sbit PA1 = IOA ^ 1;	// is alt. function INT1# see TRM Chapter 4
sbit PA2 = IOA ^ 2; // is SLOE
//sbit PA3 = IOA ^ 3;	// alt. function WU2; see TRM Chapter 6 Section 4.1
sbit PA4 = IOA ^ 4;	// is FIFOADR0 (?) don't know if it's necessary in this 2 EP config
sbit PA5 = IOA ^ 5; // is FIFOADR1 (?) don't know if it's necessary in this 2 EP config
sbit PA6 = IOA ^ 6;	// is PKTEND   (?) perhaps it could be necessary for my FPGA design
sbit PA7 = IOA ^ 7; // is FLAGD but configured as normal IO Pin


void SetupCommand(void);

/*=============================================================================
	Task Dispatcher hooks
	The following hooks are called by the task dispatcher.
=============================================================================*/

void TD_Init(void)             // Called once at startup
{
	//some FX2 CPU settings:
	CPUCS = ((CPUCS & ~bmCLKSPD) | bmCLKSPD1) ;
	CPUCS |= bmCLKOE;

	// These registers require a synchronization delay, see section 15.14 (P15-105)
  	// FIFORESET	FIFOPINPOLAR	INPKTEND
	// EPxBCH:L		EPxFIFOPFH_L	EPxAUTOINLENH:L
	// EPxFIFOCFG	EPxGPIFFLGSEL	PINFLAGSAB
	// PINFLAGSCD	EPxFIFOIE		EPxFIFOIRQ
	// GPIFIE		GPIFIRQ			UDMACRCH:L
	// GPIFADRH:L	GPIFTRIG		EPxGPIFTRIG
	// OUTPKTEND	REVCTL			GPIFTCB3
	// GPIFTCB2		GPIFTCB1		GPIFTCB0
  
  	REVCTL = 0x03;	// MUST set REVCTL.0 and REVCTL.1 to to '1'
	SYNCDELAY;
	FIFORESET = 0x80; // activate NAK all to avoid race conditions
	SYNCDELAY;
	FIFORESET = 0x02; // reset fifo buffer of EP2
	SYNCDELAY;
	FIFORESET = 0x04; // reset fifo buffer of EP4
	SYNCDELAY;
	FIFORESET = 0x06; // reset fifo buffer of EP6
	SYNCDELAY;
	FIFORESET = 0x08; // reset fifo buffer of EP8
	SYNCDELAY;
	FIFORESET = 0x00; // deactivate NAK-ALL, normal operation mode 

	/*=======================================================================
	  Now we configure IFCLK Input and slave FIFO Mode:
	  - external clock: b7 = 0 => b6, b5 have no effect
	  					b4 = 0 => do not invert IFCLK
						b3 = 0 => synchronous transmission
						b2 = 0 => no effect, only GPIF related
						b1 and b0 = 1 => activate slave FIFO mode
	=======================================================================*/

	IFCONFIG = 0x03; // 0000.0011 			  

  	// default: all endpoints have their VALID bit set
  	// default: TYPE1 = 1 and TYPE0 = 0 --> BULK  
  	// default: EP2 and EP4 DIR bits are 0 (OUT direction)
  	// default: EP6 and EP8 DIR bits are 1 (IN direction)
  	// default: EP2, EP4, EP6, and EP8 are double buffered

  	// we are not using the default values, so the following is necessary:
	// deactive EP1IN/OUT, EP4,EP8
	// configure EP2 as bulk out double buffered at 16 bit WORDWIDE, 512byte
	// configure EP6 as bulk IN quad buffered at 16bit WORDWIDE, 512byte
  	
	// REMEMBER! The PINFLAGS are arranged BACKWARDS!
	// PINFLAGSAB = [FLAGB ; FLAGA] / PINFLAGSCD = [FLAGD ; FLAGC]

	// just jusing the default acitve low structure:
	FIFOPINPOLAR = 0x00; // as it is set in defaults, no inverting

	// Configuration of EP1:
	EP1OUTCFG = 0x00;		// deactivate endpoint 1 OUT
  	EP1INCFG = 0x00;		// deactivate endpoint 1 IN
  	SYNCDELAY;            // some registers need this, see TRM section 15.14
  	
	  
	// Configuration of EP2
	EP2CFG = 0xA2;		// this is default: BULK, OUT, 512byte, double buffered
	EP2FIFOCFG = 0x11;  // enables AUTO_OUT and set WORDWIDE to 16bit
	SYNCDELAY;
	PINFLAGSAB = 0xC8;	// fixed PINFlags: FLAGA = EP2EFlag ; FLAGB = EP2FFlag
	SYNCDELAY;
	          
	// Configuration of EP4          
  	EP4CFG = 0x00;		// deactivate endpoint 4
  	SYNCDELAY;
	
	// Configuration of EP6                    
  	EP6CFG = 0xE0;		// BULK, IN, 512byte, quad buffered
	//EP6FIFOCFG = 0x4D; // FIFO Full flag comes up 1 Clk earlier
	EP6FIFOCFG = 0x0D;	// enables AUTO_IN, ZEROLENIN and sets WORDWIDE @ 16bit
	SYNCDELAY;
	PINFLAGSAB = 0x0E;	// fixed PINFlags: FLAGC = EP6FFlag ; FLAGD = not used
	SYNCDELAY;
	// configuration of length to AUTO IN
	EP6AUTOINLENH = 0x02; // 	configures AUTOINLEN as transmission trigger
	EP6AUTOINLENL = 0x00; //	to 512 bytes


	PORTACFG = 0x00; 	// uses PA / FLAGD as a port pin not as a FIFO flag
	SYNCDELAY;
		
	// Configuration of EP8                    
 	EP8CFG = 0x00;		// deactivate endpoint 8
  	SYNCDELAY;
}


void TD_Poll(void)              // Called repeatedly while the device is idle
{
}

BOOL TD_Suspend(void)          // Called before the device goes into suspend mode
{
   return(TRUE);
}

BOOL TD_Resume(void)          // Called after the device resumes
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   
// Called when a Set Configuration command is received
{
   if (EZUSB_HIGHSPEED())
	{
	// ... FX2 in highspeed mode
	EP6AUTOINLENH = 0x02; // 	configures AUTOINLEN as transmission trigger
	EP6AUTOINLENL = 0x00; //	to 512 bytes
	}
	else
	{
	// ... FX2 in fullspeed mode
	EP6AUTOINLENH = 0x00; // 	configures AUTOINLEN as transmission trigger
	EP6AUTOINLENL = 0x40; //	to 64 bytes
	}
	Configuration = SETUPDAT[ 2 ];
	return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   
// Called when a Get Configuration command is received
{
   EP0BUF[ 0 ] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       
// Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[ 2 ];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       
// Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

BOOL DR_VendorCmnd(void)
{
  return(TRUE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
   SetupCommand();	// speed up, if it is 'here' -> see USB Spec.
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   /* Commented defaults:
   // whenever we get a USB reset, we should revert to full speed mode
   pConfigDscr = pFullSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
   pOtherConfigDscr = pHighSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
   */

   if ( EZUSB_HIGHSPEED() )
   { 
   	pConfigDscr = pHighSpeedConfigDscr;
	pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
	pConfigDscr = pFullSpeedConfigDscr;
	pOtherConfigDscr = pHighSpeedConfigDscr;
   }
   
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ	    	
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      //((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
      pOtherConfigDscr = pFullSpeedConfigDscr;
     // ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;
   }
   else
   {
	pConfigDscr = pFullSpeedConfigDscr;
	pOtherConfigDscr = pHighSpeedConfigDscr;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
}
void ISR_Ep1out(void) interrupt 0
{
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{
}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
}
void ISR_GpifWaveform(void) interrupt 0
{
}
void ISR_int1(void) interrupt 2 using 2
{
}
